using System;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace ReflectionIT.UserSettings
{
	/// <summary>
	/// The SerializableSetting class is used to store, serialize and deserialize a 
	/// setting. ISerializable is implemented to create an exceptionfree deserialization.
	///</summary>
	/// <remarks>
	/// Deserialization can go wrong when the Field members of an type differ in the
	/// two versions of an Assembly. You don't want to stop the process of deserialization of
	/// the complete IsolatedUserSettings. 
	/// </remarks>
	[Serializable]
	internal class SerializableSetting : ISerializable
	{
		private object _data;
		private string _key;

		public SerializableSetting(string key, object data)
		{
			_key = key;
			_data = data;
		}

		protected SerializableSetting(SerializationInfo info, StreamingContext context) {
			try {
				_key = info.GetString("key");
				byte[] b = (byte[])info.GetValue("data", typeof(Array));
				MemoryStream stream = new MemoryStream(b);
				BinaryFormatter formatter = new BinaryFormatter();
				_data = formatter.Deserialize(stream);
			} catch (SerializationException) {
				// Ignore the SerializationExceptions
			}
		}

		public object Data {
			get { return _data; }
			set { _data = value; }
		}

		public string Key {
			get { return _key; }
		}

		public void GetObjectData(SerializationInfo info, StreamingContext context) {
			info.AddValue("key", this.Key);
			if (_data != null) {
				MemoryStream stream = new MemoryStream();
				BinaryFormatter formatter = new BinaryFormatter();
				formatter.Serialize(stream, _data);
				stream.Flush();
				byte[] b = stream.ToArray();
				info.AddValue("data", b);
			}
		}
	}
}
